#ifndef __CArray__
#define __CArray__

#include <qarray.h>

template<class type> class CArray : public QArray<type>
{
public:
	CArray(){};
	CArray(int size); 
	CArray( const CArray<type> &a);
	~CArray(){};
	CArray<type> &operator=(const CArray<type> &a)
				{ return assign(a); };
	type *data()    const	{ return (type *)QArray<type>::data(); }
	uint  nrefs()   const	{ return QArray<type>::nrefs(); }
	uint  size()    const	{ return QArray<type>::size(); }
	bool  isEmpty() const	{ return QArray<type>::size() == 0; }
	bool  isNull()  const	{ return QArray<type>::data() == 0; }
	bool  resize( uint size )	{ return QArray<type>::resize(size); }
	bool  truncate( uint pos )	{ return QArray<type>::resize(pos); }
	bool  fill( const type &v, int size=-1 )
		{ return QArray<type>::fill(v,size ); }
	void  detach()		{ QArray<type>::detach(); }
	CArray<type>   copy() const
		{ CArray<type> tmp; return tmp.duplicate(*this); }
	CArray<type>& assign( const CArray<type>& a )
		{ return (CArray<type>&)QArray<type>::assign(a); }
	CArray<type>& assign( const type *a, uint n )
		{ return (CArray<type>&)QArray<type>::assign(a,n); }
	CArray<type>& duplicate( const CArray<type>& a )
		{ return (CArray<type>&)QArray<type>::duplicate(a); }
	CArray<type>& duplicate( const type *a, uint n )
		{ return (CArray<type>&)QArray<type>::duplicate(a,n); }
	CArray<type>& setRawData( const type *a, uint n )
		{ return (CArray<type>&)QArray<type>::setRawData(a,n); }
	void resetRawData( const type *a, uint n )
		{ QArray<type>::resetRawData(a,n); }
	int	 find( const type &d, uint i=0 ) const
		{ return QArray<type>::find(d,i); }
	int	 contains( const type &d ) const
		{ return QArray<type>::contains(d); }
	type& operator[]( int i ) const
		{ return QArray<type>::at(i); }
	type& at( uint i ) const
		{ return QArray<type>::at(i); }
	operator const type*() const { return (const type *)QArray<type>::data(); }
	bool operator==( const CArray<type> &a ) const { return isEqual(a); }
	bool operator!=( const CArray<type> &a ) const { return !isEqual(a); }

	bool	insert(CArray<type>*,uint index);
	bool	insert(type*,uint n,uint index);

	bool	replace(CArray<type>*,uint index);
	bool	replace(type*,uint n,uint index);

	bool	cut(uint index,uint n);
	bool	del(uint index);

	bool	get(CArray<type>*,uint index,uint n);
	bool	get(type*,uint index,uint n);

protected:
	bool	move(type*,type*,uint);
};

#endif

