#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "csample.h"
#include "machine.c"

#define SWAPBYTES(a)	((((a) & 0xff00) >> 8) + (((a) &0x00ff) << 8))

CSample::CSample()
{
	len = 0;
	rate = 0;
	name = NULL;
	loop[0] = 0;
	loop[1] = 0;
	loop[2] = 0;
	sample_mode = 0;
	data.resize(0);

	audio_dev = "/dev/dsp";
	audio = -1;

	buffer_size = 65536;
}

CSample::CSample(uint c)
{
	len = 0;
	rate = 0;
	name = NULL;
	loop[0] = 0;
	loop[1] = 0;
	loop[2] = 0;
	sample_mode = 0;
	data.resize(0);
	addChannel(c);

	audio_dev = "/dev/dsp";
	audio = -1;

	buffer_size = 65536;
}

CSample::CSample(uint c,uint mode,uint size)
{
	len = size;
	rate = 0;
	name = NULL;
	loop[0] = 0;
	loop[1] = 0;
	loop[2] = 0;
	sample_mode = mode;
	data.resize(0);
	addChannel(c);

	audio_dev = "/dev/dsp";
	audio = -1;

	buffer_size = 65536;
}

CSample::CSample(CSample *s)
{
	len = s->len;
	rate = s->rate;
	if(s->name) name = strdup(s->name);
	else name = NULL;
	loop[0] = s->loop[0];
	loop[1] = s->loop[1];
	loop[2] = s->loop[2];
	sample_mode = s->sample_mode;
	data.resize(0);
	addChannel(s->data.size());
	set(0,s->len,s);
}

CSample::~CSample()
{
	unsigned int	t;

	if(name) free(name);
	name = NULL;

	for(t = 0;t < data.size();t++)
	{
		delete data[t].d8;
	}
	data.resize(0);
}

void CSample::setName(char *n)
{
	name = strdup(n);
}

char*	CSample::getName()
{
	return name;
}

void	CSample::setRate(int r)
{
	rate = r;
}

int	CSample::getRate()
{
	return rate;
}

void	CSample::setPitch(ushort p)
{
	pitch = p;
}

ushort	CSample::getPitch()
{
	return pitch;
}

void	CSample::setPitchRange(ushort pmin,ushort pmax)
{
	pitch_min = pmin;
	pitch_max = pmax;
}

ushort	CSample::getPitchMin()
{
	return pitch_min;
}

ushort	CSample::getPitchMax()
{
	return pitch_max;
}

unsigned int CSample::getLength()
{
	return len;
}

int	CSample::getNChannel()
{
	return data.size();
}

int	CSample::addChannel()
{
	uint		t;

	data.resize(data.size() + 1);
	//if(data == NULL) return 0;
	if(isType8())
	{
		data[data.size() - 1].d8 = new CArray<char>();
		data[data.size() - 1].d8->resize(len);
		for(t = 0;t < len;t++) data[data.size() - 1].d8->at(t) = 0;
	}
	if(isType16())
	{
		data[data.size() - 1].d16 = new CArray<short>();
		data[data.size() - 1].d16->resize(len);
		for(t = 0;t < len;t++) data[data.size() - 1].d16->at(t) = 0;
	}
	if(isType32())
	{
		data[data.size() - 1].d32 = new CArray<int>();
		data[data.size() - 1].d32->resize(len);
		for(t = 0;t < len;t++) data[data.size() - 1].d32->at(t) = 0;
	}
	if(isTypeFL())
	{
		data[data.size() - 1].dfl = new CArray<float>();
		data[data.size() - 1].dfl->resize(len);
		for(t = 0;t < len;t++) data[data.size() - 1].dfl->at(t) = 0;
	}
	if(isTypeDB())
	{
		data[data.size() - 1].ddb = new CArray<double>();
		data[data.size() - 1].ddb->resize(len);
		for(t = 0;t < len;t++) data[data.size() - 1].ddb->at(t) = 0;
	}
	return !0;
}

int	CSample::addChannel(uint n)
{
	uint		t;

	for(t = 0;t < n;t++)
	{
		if(!addChannel()) return 0;
	}

	return n;
}

int	CSample::delChannel(uint n)
{
	uint		t;

	if(n >= data.size()) return 0;

	delete data[n].d8;
	//data.del(n);
	//data.cut(n,1);
	for(t = n;t < data.size() - 1;t++)
	{
		data[t] = data[t + 1];
	}
	data.resize(data.size() - 1);

	return !0;
}
 
int	CSample::clear()
{
	uint		t;

	for(t = data.size();t > 0;t--) delChannel(0);

	if(getNChannel() == 0)
	{
		len = 0;
		data.resize(0);
	
		return !0;
	}

	return 0;
}

int	CSample::isMono()
{
	return (getNChannel() == 1);
}

int	CSample::isStereo()
{
	return (getNChannel() == 2);
}

int	CSample::isType8()
{
	return ((sample_mode & SMPFLAGS_TYPEMASK) == SMPFLAGS_TYPE_8);
}

int	CSample::isType16()
{
	return ((sample_mode & SMPFLAGS_TYPEMASK) == SMPFLAGS_TYPE_16);
}

int	CSample::isType32()
{
	return ((sample_mode & SMPFLAGS_TYPEMASK) == SMPFLAGS_TYPE_32);
}

int	CSample::isTypeFL()
{
	return ((sample_mode & SMPFLAGS_TYPEMASK) == SMPFLAGS_TYPE_FL);
}

int	CSample::isTypeDB()
{
	return ((sample_mode & SMPFLAGS_TYPEMASK) == SMPFLAGS_TYPE_DB);
}

void	CSample::convertToTypeDB()
{
	uint		c,t;

	if(isTypeDB()) return;
	if(len == 0)
	{
		sample_mode = SMPFLAGS_TYPE_DB;
		return;
	}

	if(isType8())
	{
		char		*i8;

		if((i8 = (char*)malloc(sizeof(char) * len)) == NULL) return;

	 	for(c = 0;c < data.size();c++)
		{
			get(c,0,len,i8);
			data[c].ddb->resize(len);
			for(t = 0;t < len;t++)
			{
				data[c].ddb->at(t) = CHAR2D(i8[t]);
			}
		}

		free(i8);
		sample_mode = SMPFLAGS_TYPE_DB;
	}
	if(isType16())
	{
		short		*i16;

		if((i16 = (short*)malloc(sizeof(short) * len)) == NULL) return;

	 	for(c = 0;c < data.size();c++)
		{
			get(c,0,len,i16);
			data[c].ddb->resize(len);
			for(t = 0;t < len;t++)
			{
				data[c].ddb->at(t) = SHORT2D(i16[t]);
			}
		}

		free(i16);
		sample_mode = SMPFLAGS_TYPE_DB;
	}
	if(isType32())
	{
		int		*i32;

		if((i32 = (int*)malloc(sizeof(int) * len)) == NULL) return;

	 	for(c = 0;c < data.size();c++)
		{
			get(c,0,len,i32);
			data[c].ddb->resize(len);
			for(t = 0;t < len;t++)
			{
				data[c].ddb->at(t) = INT2D(i32[t]);
			}
		}

		free(i32);
		sample_mode = SMPFLAGS_TYPE_DB;
	}
	if(isTypeFL())
	{
		float		*fl;

		if((fl = (float*)malloc(sizeof(float) * len)) == NULL) return;

	 	for(c = 0;c < data.size();c++)
		{
			get(c,0,len,fl);
			data[c].ddb->resize(len);
			for(t = 0;t < len;t++)
			{
				data[c].ddb->at(t) = FLOAT2D(fl[t]);
			}
		}

		free(fl);
		sample_mode = SMPFLAGS_TYPE_DB;
	}
	
	
}

void	CSample::convertTypeDBToType8()
{
	uint		c,t;
	double		*d;

	if(!isTypeDB()) return;
	if(len == 0)
	{
		sample_mode = SMPFLAGS_TYPE_8;
		return;
	}

	if((d = (double*)malloc(sizeof(double) * len)) == NULL) return;

 	for(c = 0;c < data.size();c++)
	{
		get(c,0,len,d);
		data[c].d8->resize(len);
		for(t = 0;t < len;t++)
		{
			data[c].d8->at(t) = D2CHAR(d[t]);
		}
	}

	free(d);
	sample_mode = SMPFLAGS_TYPE_8;
}

void	CSample::convertTypeDBToType16()
{
	uint		c,t;
	double		*d;

	if(!isTypeDB()) return;
	if(len == 0)
	{
		sample_mode = SMPFLAGS_TYPE_16;
		return;
	}

	if((d = (double*)malloc(sizeof(double) * len)) == NULL) return;

 	for(c = 0;c < data.size();c++)
	{
		get(c,0,len,d);
		data[c].d16->resize(len);
		for(t = 0;t < len;t++)
		{
			data[c].d16->at(t) = D2SHORT(d[t]);
		}
	}

	free(d);
	sample_mode = SMPFLAGS_TYPE_16;
}

void	CSample::convertTypeDBToType32()
{
	uint		c,t;
	double		*d;

	if(!isTypeDB()) return;
	if(len == 0)
	{
		sample_mode = SMPFLAGS_TYPE_32;
		return;
	}

	if((d = (double*)malloc(sizeof(double) * len)) == NULL) return;

 	for(c = 0;c < data.size();c++)
	{
		get(c,0,len,d);
		data[c].d32->resize(len);
		for(t = 0;t < len;t++)
		{
			data[c].d32->at(t) = D2INT(d[t]);
		}
	}

	free(d);
	sample_mode = SMPFLAGS_TYPE_32;
}

void	CSample::convertTypeDBToTypeFL()
{
	uint		c,t;
	double		*d;

	if(!isTypeDB()) return;
	if(len == 0)
	{
		sample_mode = SMPFLAGS_TYPE_FL;
		return;
	}

	if((d = (double*)malloc(sizeof(double) * len)) == NULL) return;

 	for(c = 0;c < data.size();c++)
	{
		get(c,0,len,d);
		data[c].dfl->resize(len);
		for(t = 0;t < len;t++)
		{
			data[c].dfl->at(t) = D2FLOAT(d[t]);
		}
	}

	free(d);
	sample_mode = SMPFLAGS_TYPE_FL;
}

int	CSample::resize(uint size)
{
	uint		t;

	if(size == len) return !0;
	for(t = 0;t < data.size();t++)
	{
		if(isType8())
		{
			data[t].d8->resize(size);
		}
		if(isType16())
		{
			data[t].d16->resize(size);
		}
		if(isType32())
		{
			data[t].d32->resize(size);
		}
		if(isTypeFL())
		{
			data[t].dfl->resize(size);
		}
		if(isTypeDB())
		{
			data[t].ddb->resize(size);
		}
	}
	len = size;

	return !0;
}

int	CSample::get(uint c,uint pos,uint l,char *d8)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			d8[t] = data[c].d8->at(t + pos);
		}
		if(isType16())
		{
			d8[t] = D2CHAR(SHORT2D(data[c].d16->at(t + pos)));
		}
		if(isType32())
		{
			d8[t] = D2CHAR(INT2D(data[c].d32->at(t + pos)));
		}
		if(isTypeFL())
		{
			d8[t] = D2CHAR(FLOAT2D(data[c].dfl->at(t + pos)));
		}
		if(isTypeDB())
		{
			d8[t] = D2CHAR(data[c].ddb->at(t + pos));
		}
	}

	return !0;
}

int	CSample::get(uint c,uint pos,uint l,short *d16)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			d16[t] = D2SHORT(CHAR2D(data[c].d8->at(t + pos)));
		}
		if(isType16())
		{
			d16[t] = data[c].d16->at(t + pos);
		}
		if(isType32())
		{
			d16[t] = D2SHORT(INT2D(data[c].d32->at(t + pos)));
		}
		if(isTypeFL())
		{
			d16[t] = D2SHORT(FLOAT2D(data[c].dfl->at(t + pos)));
		}
		if(isTypeDB())
		{
			d16[t] = D2SHORT(data[c].ddb->at(t + pos));
		}
	}

	return !0;
}

int	CSample::get(uint c,uint pos,uint l,int *d32)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			d32[t] = D2INT(CHAR2D(data[c].d8->at(t + pos)));
		}
		if(isType16())
		{
			d32[t] = D2INT(SHORT2D(data[c].d16->at(t + pos)));
		}
		if(isType32())
		{
			d32[t] = D2INT(INT2D(data[c].d32->at(t + pos)));
		}
		if(isTypeFL())
		{
			d32[t] = D2INT(FLOAT2D(data[c].dfl->at(t + pos)));
		}
		if(isTypeDB())
		{
			d32[t] = D2INT(data[c].ddb->at(t + pos));
		}
	}

	return !0;
}

int	CSample::get(uint c,uint pos,uint l,float *dfl)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			dfl[t] = D2FLOAT(CHAR2D(data[c].d8->at(t + pos)));
		}
		if(isType16())
		{
			dfl[t] = D2FLOAT(SHORT2D(data[c].d16->at(t + pos)));
		}
		if(isType32())
		{
			dfl[t] = D2FLOAT(INT2D(data[c].d32->at(t + pos)));
		}
		if(isTypeFL())
		{
			dfl[t] = D2FLOAT(FLOAT2D(data[c].dfl->at(t + pos)));
		}
		if(isTypeDB())
		{
			dfl[t] = D2FLOAT(data[c].ddb->at(t + pos));
		}
	}

	return !0;
}

int	CSample::get(uint c,uint pos,uint l,double *ddb)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			ddb[t] = CHAR2D(data[c].d8->at(t + pos));
		}
		if(isType16())
		{
			ddb[t] = SHORT2D(data[c].d16->at(t + pos));
		}
		if(isType32())
		{
			ddb[t] = INT2D(data[c].d32->at(t + pos));
		}
		if(isTypeFL())
		{
			ddb[t] = FLOAT2D(data[c].dfl->at(t + pos));
		}
		if(isTypeDB())
		{
			ddb[t] = data[c].ddb->at(t + pos);
		}
	}

	return !0;
}

int	CSample::get(uint c,uint pos,uint l,CSample *s)
{
	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	s->clear();
	s->resize(l);
	s->addChannel();
	if(s->isType8())
	{
		get(c,pos,l,s->data[c].d8->data());
	}
	if(s->isType16())
	{
		get(c,pos,l,s->data[c].d16->data());
	}
	if(s->isType32())
	{
		get(c,pos,l,s->data[c].d32->data());
	}
	if(s->isTypeFL())
	{
		get(c,pos,l,s->data[c].dfl->data());
	}
	if(s->isTypeDB())
	{
		get(c,pos,l,s->data[c].ddb->data());
	}

	return !0;
}

int	CSample::get(uint pos,uint l,CSample *s)
{
	uint		t;

	// if(s->data.size() > data.size()) return 0;
	if(pos + l > len) return 0;
	s->clear();
	s->resize(l);
	s->addChannel(data.size());
	for(t = 0;t < data.size();t++)
	{
		if(s->isType8())
		{
			get(t,pos,l,s->data[t].d8->data());
		}
		if(s->isType16())
		{
			get(t,pos,l,s->data[t].d16->data());
		}
		if(s->isType32())
		{
			get(t,pos,l,s->data[t].d32->data());
		}
		if(s->isTypeFL())
		{
			get(t,pos,l,s->data[t].dfl->data());
		}
		if(s->isTypeDB())
		{
			get(t,pos,l,s->data[t].ddb->data());
		}
	}

	return !0;
}

int	CSample::set(uint c,uint pos,uint l,char *d8)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			data[c].d8->at(t + pos) = D2CHAR(CHAR2D(d8[t]));
		}
		if(isType16())
		{
			data[c].d16->at(t + pos) = D2SHORT(CHAR2D(d8[t]));
		}
		if(isType32())
		{
			data[c].d32->at(t + pos) = D2INT(CHAR2D(d8[t]));
		}
		if(isTypeFL())
		{
			data[c].dfl->at(t + pos) = D2FLOAT(CHAR2D(d8[t]));
		}
		if(isTypeDB())
		{
			data[c].ddb->at(t + pos) = CHAR2D(d8[t]);
		}
	}

	return !0;
}

int	CSample::set(uint c,uint pos,uint l,short *d16)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			data[c].d8->at(t + pos) = D2CHAR(SHORT2D(d16[t]));
		}
		if(isType16())
		{
			data[c].d16->at(t + pos) = D2SHORT(SHORT2D(d16[t]));
		}
		if(isType32())
		{
			data[c].d32->at(t + pos) = D2INT(SHORT2D(d16[t]));
		}
		if(isTypeFL())
		{
			data[c].dfl->at(t + pos) = D2FLOAT(SHORT2D(d16[t]));
		}
		if(isTypeDB())
		{
			data[c].ddb->at(t + pos) = SHORT2D(d16[t]);
		}
	}

	return !0;
}

int	CSample::set(uint c,uint pos,uint l,int *d32)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			data[c].d8->at(t + pos) = D2CHAR(INT2D(d32[t]));
		}
		if(isType16())
		{
			data[c].d16->at(t + pos) = D2SHORT(INT2D(d32[t]));
		}
		if(isType32())
		{
			data[c].d32->at(t + pos) = D2INT(INT2D(d32[t]));
		}
		if(isTypeFL())
		{
			data[c].dfl->at(t + pos) = D2FLOAT(INT2D(d32[t]));
		}
		if(isTypeDB())
		{
			data[c].ddb->at(t + pos) = INT2D(d32[t]);
		}
	}

	return !0;
}

int	CSample::set(uint c,uint pos,uint l,float *dfl)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			data[c].d8->at(t + pos) = D2CHAR(FLOAT2D(dfl[t]));
		}
		if(isType16())
		{
			data[c].d16->at(t + pos) = D2SHORT(FLOAT2D(dfl[t]));
		}
		if(isType32())
		{
			data[c].d32->at(t + pos) = D2INT(FLOAT2D(dfl[t]));
		}
		if(isTypeFL())
		{
			data[c].dfl->at(t + pos) = D2FLOAT(FLOAT2D(dfl[t]));
		}
		if(isTypeDB())
		{
			data[c].ddb->at(t + pos) = INT2D(dfl[t]);
		}
	}

	return !0;
}

int	CSample::set(uint c,uint pos,uint l,double *ddb)
{
	uint		t;

	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < l;t++)
	{
		if(isType8())
		{
			data[c].d8->at(t + pos) = D2CHAR(ddb[t]);
		}
		if(isType16())
		{
			data[c].d16->at(t + pos) = D2SHORT(ddb[t]);
		}
		if(isType32())
		{
			data[c].d32->at(t + pos) = D2INT(ddb[t]);
		}
		if(isTypeFL())
		{
			data[c].dfl->at(t + pos) = D2FLOAT(ddb[t]);
		}
		if(isTypeDB())
		{
			data[c].ddb->at(t + pos) = ddb[t];
		}
	}

	return !0;
}

int	CSample::set(uint c,uint pos,uint l,CSample *s)
{
	if(c > data.size()) return 0;
	if(pos + l > len) return 0;
	if(s->isType8())
	{
		set(c,pos,l,s->data[0].d8->data());
	}
	if(isType16())
	{
		set(c,pos,l,s->data[0].d16->data());
	}
	if(isType32())
	{
		set(c,pos,l,s->data[0].d32->data());
	}
	if(isTypeFL())
	{
		set(c,pos,l,s->data[0].dfl->data());
	}
	if(isTypeDB())
	{
		set(c,pos,l,s->data[0].ddb->data());
	}

	return !0;
}

int	CSample::set(uint c,uint pos,uint l,CSample *s,uint sc)
{
	if(c > data.size()) return 0;
	if(sc > s->data.size()) return 0;
	if(pos + l > len) return 0;
	if(s->isType8())
	{
		set(c,pos,l,s->data[sc].d8->data());
	}
	if(s->isType16())
	{
		set(c,pos,l,s->data[sc].d16->data());
	}
	if(s->isType32())
	{
		set(c,pos,l,s->data[sc].d32->data());
	}
	if(s->isTypeFL())
	{
		set(c,pos,l,s->data[sc].dfl->data());
	}
	if(s->isTypeDB())
	{
		set(c,pos,l,s->data[sc].ddb->data());
	}

	return !0;
}

int	CSample::set(uint pos,uint l,CSample *s)
{
	uint		t;

	if(data.size() != s->data.size()) return 0;
	if(pos + l > len) return 0;
	for(t = 0;t < data.size();t++)
	{
		if(s->isType8())
		{
			set(t,pos,l,s->data[t].d8->data());
		}
		if(s->isType16())
		{
			set(t,pos,l,s->data[t].d16->data());
		}
		if(s->isType32())
		{
			set(t,pos,l,s->data[t].d32->data());
		}
		if(s->isTypeFL())
		{
			set(t,pos,l,s->data[t].dfl->data());
		}
		if(s->isTypeDB())
		{
			set(t,pos,l,s->data[t].ddb->data());
		}
	}

	return !0;
}

int	CSample::move(uint dest,uint src,int l)
{
	uint		c;

	if(l <= 0)
	{
		printf("CSample: move: l out of range (%i)!\n",l);
		return 0;
	}
	if(dest + l > len)
	{
		printf("CSample: move: dest + l out of range (%i,%i)!\n",dest + l,len);
		return 0;
	}
	if(src + l > len)
	{
		printf("CSample: move: src + l out of range!\n");
		return 0;
	}

	if(isType8())
	{
		for(c = 0;c < data.size();c++)
			memmove(data[c].d8 + dest,data[c].d8 + src,l * sizeof(char));
	}
	else if(isType16())
	{
		for(c = 0;c < data.size();c++)
			memmove(data[c].d16 + dest,data[c].d16 + src,l * sizeof(short));
	}
	else if(isType32())
	{
		for(c = 0;c < data.size();c++)
			memmove(data[c].d32 + dest,data[c].d32 + src,l * sizeof(int));
	}
	else if(isTypeFL())
	{
		for(c = 0;c < data.size();c++)
			memmove(data[c].dfl + dest,data[c].dfl + src,l * sizeof(float));
	}
	else if(isTypeDB())
	{
		for(c = 0;c < data.size();c++)
			memmove(data[c].ddb + dest,data[c].ddb + src,l * sizeof(double));
	}

	return !0;
}

int	CSample::insert(uint pos,uint l,CSample *s)
{
	uint		t;

	if(s->data.size() != data.size()) return 0;
	if(pos > len) return 0;
	//if(l > s->len) return 0;
	printf("resize: %i to %i\n",len,l + len);
	resize(len + l);
	printf("move: %i,%i,%i\n",l + pos,pos,len - l - pos - 1);
	//move(pos + l,pos,len - l - pos - 1);
	move(pos + l,pos,len - l - pos - 1);
	for(t = 0;t < data.size();t++)
	{
		printf("set: %i,%i\n",pos,l);
		set(t,pos,l,s,t);
	}

	return !0;
}

int	CSample::insert(uint pos,CSample *s)
{
	uint		t,l;

	if(s == NULL) return 0;
	l = s->getLength();
	if(s->data.size() != data.size()) return 0;
	if(pos > len) return 0;
	//if(l > s->len) return 0;
	printf("resize: %i to %i\n",len,l + len);
	resize(len + l);
	printf("move: %i,%i,%i\n",l + pos,pos,len - l - pos - 1);
	//move(pos + l,pos,len - l - pos - 1);
	move(pos + l,pos,len - l - pos - 1);
	for(t = 0;t < data.size();t++)
	{
		printf("set: %i,%i\n",pos,l);
		set(t,pos,l,s,t);
	}

	return !0;
}

int	CSample::cut(uint pos,uint l,CSample *s = NULL)
{
	if(pos + l > len) return 0;
	if(s != NULL)
	{
		if(s->data.size() != data.size()) return 0;
		if(!get(pos,l,s)) return 0;
	}
	move(pos,pos + l,len - (pos + l + 1));
	resize(len - l);

	return !0;
}

int CSample::loadWave(char* namepath)
{
	FILE		*fp;
	char		buffer[256];
	uint		t,l;

        if((fp = fopen(namepath,"rb")) == NULL) return -1;
	fread(buffer,8,1,fp);
	if(strncmp(buffer,"RIFF",4) != 0)
	{
		fclose(fp);
		return -2;
	}
	fread(buffer,4,1,fp);
	if(strncmp(buffer,"WAVE",4) != 0)
	{
		fclose(fp);
		return -3;
	}
	if(fread(buffer,8,1,fp) == 0) return -4;
	l = make_le4(*(int*)(buffer + 4));
	if(strncmp(buffer,"fmt ",4) == 0)
	{
		fread(buffer,l,1,fp);
		rate = make_le4(*(int*)(buffer + 4));
		if(make_le2(*(short*)(buffer + 14)) == 8)
		{
			sample_mode = SMPFLAGS_TYPE_8;
		}
		else
		{
			sample_mode = SMPFLAGS_TYPE_16;
		}
		if(make_le2(*(short*)(buffer + 2)) - 1)
		{
			clear();
			addChannel(2);
		}
		else
		{
			clear();
			addChannel(1);
		}
	}
	else return -4;
	if(fread(buffer,8,1,fp) == 0) return -4;
	l = make_le4(*(int*)(buffer + 4));
	if(strncmp(buffer,"data",4) == 0)
	{
		if(isStereo())
		{
			if(isType8())
			{
				char*	buffer = (char*)malloc(l);
				char*	bl = (char*)malloc(l / 2);
				char*	br = (char*)malloc(l / 2);

				fread(buffer,l,1,fp);
				for(t = 0;t < l / 2;t++)
				{
					bl[t] = 127 - buffer[t * 2];
					br[t] = 127 - buffer[t * 2 + 1];
				}
				resize(l / 2);
				set(0,0,l / 2,bl);
				set(1,0,l / 2,br);
				free(buffer);
				free(bl);
				free(br);
			}
			if(isType16())
			{
				short*	buffer = (short*)malloc(l);
				short*	bl = (short*)malloc(l / 2);
				short*	br = (short*)malloc(l / 2);

				fread(buffer,l,1,fp);
				for(t = 0;t < l / 4;t++)
				{
					bl[t] = 32767 - buffer[t * 2];
					br[t] = 32767 - buffer[t * 2 + 1];
				}
				resize(l / 4);
				set(0,0,l / 4,bl);
				set(1,0,l / 4,br);
				free(buffer);
				free(bl);
				free(br);
			}
		}
		else
		{
			if(isType8())
			{
				char*	buffer = (char*)malloc(l);

				fread(buffer,l,1,fp);
				for(t = 0;t < l;t++)
				{
					buffer[t] = 127 - buffer[t];
					//printf("%i\n",buffer[t]);
				}
				resize(l);
				set(0,0,l,buffer);
				free(buffer);
			}
			if(isType16())
			{
				short*	buffer = (short*)malloc(l);

				fread(buffer,l,1,fp);
				for(t = 0;t < l / 2;t++)
				{
					buffer[t] = 32767 - buffer[t];
				}
				resize(l / 2);
				set(0,0,l / 2,buffer);
				free(buffer);
			}
		}
	}
	else return -4;

	fclose(fp);

	return 0;
}

int CSample::saveWave(char *namepath)
{
	FILE		*fp;
	char		buffer[256];
	uint		t,l;

	if((fp = fopen(namepath,"wb")) == NULL) return -1;
	sprintf(buffer,"RIFF");
	l = 1;
	if(isStereo()) l *= 2;
	if(isType16()) l *= 2;
	l = 12 + 8 + 16 + 8 + getLength() * l;
	writestr4(buffer + 4,make_le4(l));
	sprintf(buffer + 8,"WAVE");
	sprintf(buffer + 12,"fmt ");
	writestr4(buffer + 16,make_le4(16));
	writestr2(buffer + 20,make_le2(1));
	if(isMono()) writestr2(buffer + 22,1);
	if(isStereo()) writestr2(buffer + 22,2);
	writestr4(buffer + 24,make_le4(getRate()));
	l = 1;
	if(isStereo()) l *= 2;
	if(isType16()) l *= 2;
	writestr4(buffer + 28,make_le4(getRate() * l));
	if(isType8()) writestr2(buffer + 32,make_le2(1));
	if(isType16()) writestr2(buffer + 32,make_le2(2));
	if(isType8()) writestr2(buffer + 34,make_le2(8));
	if(isType16()) writestr2(buffer + 34,make_le2(16));
	sprintf(buffer + 36,"data");
	l = getLength();
	if(isStereo()) l *= 2;
	if(isType16()) l *= 2;
	l = (l & 1 ? l + 1 : l);
	writestr4(buffer + 40,make_le4(l));
	fwrite(buffer,44,1,fp);
	for(t = 0;t < getLength();t++)
	{
		if(isMono())
		{
			if(isType8())
			{
				char	d;

				d = 127 - data[0].d8->at(t);
				fwrite(&d,sizeof(char),1,fp);
			}
			if(isType16())
			{
				short	d;

				d = 32767 - data[0].d16->at(t);
				fwrite(&d,sizeof(short),1,fp);
			}
		}
		if(isStereo())
		{
			if(isType8())
			{
				char	d;

				d = 127 - data[0].d8->at(t);
				fwrite(&d,sizeof(char),1,fp);
				d = 127 - data[1].d8->at(t);
				fwrite(&d,sizeof(char),1,fp);
			}
			if(isType16())
			{
				short	d;

				d = 32767 - data[0].d16->at(t);
				fwrite(&d,sizeof(short),1,fp);
				d = 32767 - data[1].d16->at(t);
				fwrite(&d,sizeof(short),1,fp);
			}
		}
	}
	if(getLength() & 1 && isType8())
	{
		char	d;

		d = 127 - data[0].d8->at(t);
		fwrite(&d,sizeof(char),1,fp);
	}
	fclose(fp);

	return 0;
}

void	CSample::setSample(CSample *s)
{
	if(this == s)
	{
		printf("same sample\n");
		return;
	}
	clear();
	len = s->len;
	rate = s->rate;
	if(s->name) name = strdup(s->name);
	else name = NULL;
	loop[0] = s->loop[0];
	loop[1] = s->loop[1];
	loop[2] = s->loop[2];
	sample_mode = s->sample_mode;
	addChannel(s->data.size());
	set(0,s->len,s);
}

void	CSample::equalize(CSample *s)
{
	if(this == s)
	{
		printf("same sample\n");
		return;
	}
	clear();
	//len = s->len;
	rate = s->rate;
	if(name) free(name);
	if(s->name) name = strdup(s->name);
	else name = NULL;
	loop[0] = s->loop[0];
	loop[1] = s->loop[1];
	loop[2] = s->loop[2];
	sample_mode = s->sample_mode;
	addChannel(s->data.size());
}

int	CSample::playback(uint s,uint e)
{
	current_pos = start_pos = MAX(s,0);
	end_pos = MIN(e,len);

	dsp_speed = rate;
	if(isType8()) dsp_samplesize = 8;
	else dsp_samplesize = 16;
	if(isMono()) dsp_stereo = 0;
	else dsp_stereo = 1;

	audio = open (audio_dev, O_WRONLY, 0);
	if(audio == -1) 
	{
		return -1;
	}
	if(ioctl(audio,SNDCTL_DSP_SPEED,&dsp_speed) == -1)
	{
		close(audio);
		audio = -1;
		return -2;
	}
	if(ioctl(audio,SNDCTL_DSP_SAMPLESIZE,&dsp_samplesize) == -1)
	{
		close(audio);
		audio = -1;
		return -3;
	}
	if(ioctl(audio,SNDCTL_DSP_STEREO,&dsp_stereo) == -1)
	{
		close(audio);
		audio = -1;
		return -4;
	}
	ioctl(audio,SNDCTL_DSP_GETBLKSIZE,&buffer_size);
	if((buffer = malloc(buffer_size)) == NULL)
	{
		close(audio);
		audio = -1;
		return -4;
	}
	if((lb = malloc(buffer_size)) == NULL)
	{
		free(buffer);
		close(audio);
		audio = -1;
		return -4;
	}
	if((rb = malloc(buffer_size)) == NULL)
	{
		free(buffer);
		free(lb);
		close(audio);
		audio = -1;
		return -4;
	}

	spb = buffer_size / (dsp_samplesize / 8 * (dsp_stereo + 1));

	return 0;
}

int	CSample::playbackevent()
{
	uint		t,i;

	t = current_pos;
	if(t > end_pos)
	{
		close(audio);
		audio = -1;
		free(buffer);
		free(lb);
		free(rb);
	
		return 0;
	}
	else
	{
		if(dsp_samplesize == 8)
		{
			get(0,t,buffer_size,(char*)lb);
			if(dsp_stereo) get(1,t,buffer_size,(char*)rb);
		}
		else
		{
			get(0,t,buffer_size / 2,(short*)lb);
			if(dsp_stereo) get(1,t,buffer_size / 2,(short*)rb);
		}
		for(i = 0;i < spb;i++)
		{
			if(t + i > end_pos)
			{
				for(;i < spb;i++)
				{
					if(dsp_samplesize == 8)
					{
						if(dsp_stereo) 
						{
							((char*)buffer)[i * 2] = 127;
							((char*)buffer)[i * 2 + 1] = 127;
						}
						else ((char*)buffer)[i] = 127;
					}
					else
					{
						if(dsp_stereo) 
						{
							((short*)buffer)[i * 2] = 0;
							((short*)buffer)[i * 2 + 1] = 0;
						}
						else ((short*)buffer)[i] = 0;
					}
				}
				write(audio,buffer,spb);
				close(audio);
				free(buffer);
				free(lb);
				free(rb);
				audio = -1;

				return 0;
			}
			if(dsp_samplesize == 8)
			{
				if(dsp_stereo) 
				{
					((char*)buffer)[i * 2] = 127 - ((char*)lb)[i];
					((char*)buffer)[i * 2 + 1] = 127 -((char*)rb)[i];
				}
				else ((char*)buffer)[i] = 127 - ((char*)lb)[i];
			}
			else
			{
				if(dsp_stereo) 
				{
					((short*)buffer)[i * 2] =
						(((short*)lb)[i]);
					((short*)buffer)[i * 2 + 1] = 
						(((short*)rb)[i]);
				}
				else ((short*)buffer)[i] = 
					(((short*)lb)[i]);
			}
		}
		if(write(audio,buffer,buffer_size) != buffer_size)
		{
			close(audio);
			audio = -1;
			free(buffer);
			free(lb);
			free(rb);
			return -5;
		}
	}

	current_pos += spb;

	return 0;
}

int	CSample::playback(double s,double e)
{
	return playback(s * rate,e * rate);
}

int	CSample::playback()
{
	return playback(0,len);
}

int	CSample::play(uint s,uint e)
{
	uint		t,i;

	dsp_speed = rate;
	if(isType8()) dsp_samplesize = 8;
	else dsp_samplesize = 16;
	if(isMono()) dsp_stereo = 0;
	else dsp_stereo = 1;

	audio = open (audio_dev, O_WRONLY, 0);
	if(audio == -1) 
	{
		return -1;
	}
	if(ioctl(audio,SNDCTL_DSP_SPEED,&dsp_speed) == -1)
	{
		close(audio);
		audio = -1;
		return -2;
	}
	if(ioctl(audio,SNDCTL_DSP_SAMPLESIZE,&dsp_samplesize) == -1)
	{
		close(audio);
		audio = -1;
		return -3;
	}
	if(ioctl(audio,SNDCTL_DSP_STEREO,&dsp_stereo) == -1)
	{
		close(audio);
		audio = -1;
		return -4;
	}
	ioctl(audio,SNDCTL_DSP_GETBLKSIZE,&buffer_size);
	if((buffer = malloc(buffer_size)) == NULL)
	{
		close(audio);
		audio = -1;
		return -4;
	}
	if((lb = malloc(buffer_size)) == NULL)
	{
		free(buffer);
		close(audio);
		audio = -1;
		return -4;
	}
	if((rb = malloc(buffer_size)) == NULL)
	{
		free(buffer);
		free(lb);
		close(audio);
		audio = -1;
		return -4;
	}

	spb = buffer_size / (dsp_samplesize / 8 * (dsp_stereo + 1));

	for(t = s;t < e;t += spb)
	{
		current_pos = t;
		if(dsp_samplesize == 8)
		{
			get(0,t,buffer_size,(char*)lb);
			if(dsp_stereo) get(1,t,buffer_size,(char*)rb);
		}
		else
		{
			get(0,t,buffer_size / 2,(short*)lb);
			if(dsp_stereo) get(1,t,buffer_size / 2,(short*)rb);
		}
		for(i = 0;i < spb;i++)
		{
			if(t + i > e)
			{
				write(audio,buffer,i);
				close(audio);
				free(buffer);
				free(lb);
				free(rb);
				audio = -1;

				return 0;
			}
			if(dsp_samplesize == 8)
			{
				if(dsp_stereo) 
				{
					((char*)buffer)[i * 2] = ((char*)lb)[i];
					((char*)buffer)[i * 2 + 1] = ((char*)rb)[i];
				}
				else ((char*)buffer)[i] = ((char*)lb)[i];
			}
			else
			{
				if(dsp_stereo) 
				{
					((short*)buffer)[i * 2] = ((short*)lb)[i];
					((short*)buffer)[i * 2 + 1] = ((short*)rb)[i];
				}
				else ((short*)buffer)[i] = ((short*)lb)[i];
			}
		}
		if(write(audio,buffer,buffer_size) != buffer_size)
		{
			close(audio);
			audio = -1;
			free(buffer);
			free(lb);
			free(rb);
			return -5;
		}
	}

	close(audio);
	audio = -1;
	free(buffer);
	free(lb);
	free(rb);

	return 0;
}

int	CSample::play(double s,double e)
{
	return play(s * rate,e * rate);
}

int	CSample::play()
{
	return play(0,len);
}

void	CSample::stop()
{
	close(audio);
	audio = -1;
}

uint	CSample::currentPlayPos()
{
	return current_pos;
}

double	CSample::currentPlayTime()
{
	return (double)current_pos / (double)rate;
}

int	CSample::playing()
{
	return (audio != -1);
}

QString	CSample::getAudioDevice()
{
	return audio_dev;
}

void	CSample::setAudioDevice(QString ad)
{
	audio_dev = ad;
}


#ifdef TEST_KSAMPLE

#include <stdio.h>

int	main(int argc,char **argv)
{
	CSample		s((uint)0);

	if(argc < 2) return 0;

	s.loadWave(argv[1]);

	printf("Wave:    %s\n",argv[1]);
	printf("Channel: %i\n",s.getNChannel());
	printf("Length:  %i\n",s.getLength());
	printf("Bits:    %i\n",(s.isType16()?16:8));

	return 1;
}

#endif

