#ifndef __CSAMPLE_H__
#define __CSAMPLE_H__

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>

#include <sys/soundcard.h>


#include <qstring.h>


#include "machine.h"
#include "carray.h"

#ifndef NULL
#define NULL 0
#endif

#define	INT2D(a)	((double)(a) / 214783648.0)
#define	SHORT2D(a)	((double)(a) / 32768.0)
#define	CHAR2D(a)	((double)(a) / 128.0)
#define	FLOAT2D(a)	((double)(a))

#define CUT(a)		(MIN(1,MAX(-1,(a))))
#define	D2CHAR(a)	((char)(CUT(a) * 127.0))
#define	D2SHORT(a)	((short)(CUT(a) * 32767.0))
#define	D2INT(a)	((int)(CUT(a) * 214783647.0))
#define	D2FLOAT(a)	((float)(a))

enum	{
		SMPFLAGS_TYPE_8 = 0,
		SMPFLAGS_TYPE_16,
		SMPFLAGS_TYPE_32,
		SMPFLAGS_TYPE_FL,
		SMPFLAGS_TYPE_DB,
		SMPFLAGS_TYPEMASK = 1 | 2 | 3 | 4
	};
	
#ifndef MIN
#define MIN(a,b)	((a)<(b)?(a):(b))
#endif

#ifndef MAX
#define MAX(a,b)	((a)>(b)?(a):(b))
#endif

#ifndef uint
typedef unsigned int uint;
#endif

#ifndef ushort
typedef unsigned short ushort;
#endif


typedef union
{
	CArray<char>	*d8;
	CArray<short>	*d16;
	CArray<int>	*d32;
	CArray<float>	*dfl;
	CArray<double>	*ddb;
}
sampledata;

class CSample
{
protected:
	uint			len;
	uint			rate;
	ushort			pitch;
	ushort			pitch_min,pitch_max;
	char			*name;
	uint			loop[3];
	uint			sample_mode;
	CArray<sampledata>	data;

public:
	CSample();
	CSample(uint c);
	CSample(uint c,uint mode,uint size);
	CSample(CSample *s);
	~CSample();

	void		setName(char *n);
	char*		getName();
	void		setRate(int r);
	int		getRate();
	void		setPitch(ushort p);
	ushort		getPitch();
	void		setPitchRange(ushort min,ushort max);
	ushort		getPitchMin();
	ushort		getPitchMax();
	uint		getLength();

	int		isMono();
	int		isStereo();
	int		getNChannel();

	int		addChannel();
	int		addChannel(uint n);
	int		delChannel(uint n);
	int		clear();

	int		isType8();
	int		isType16();
	int		isType32();
	int		isTypeFL();
	int		isTypeDB();

	uint		getType() { return sample_mode;};

	void		convertToTypeDB();

	void		convertTypeDBToType8();
	void		convertTypeDBToType16();
	void		convertTypeDBToType32();
	void		convertTypeDBToTypeFL();

	int		resize(uint size);

	int		get(uint c,uint pos,uint len,char *d8);
	int		get(uint c,uint pos,uint len,short *d16);
	int		get(uint c,uint pos,uint len,int *d32);
	int		get(uint c,uint pos,uint len,float *dfl);
	int		get(uint c,uint pos,uint len,double *ddb);
	int		get(uint c,uint pos,uint len,CSample *s);
	int		get(uint pos,uint len,CSample *s);

	int		set(uint c,uint pos,uint len,char *d8);
	int		set(uint c,uint pos,uint len,short *d16);
	int		set(uint c,uint pos,uint len,int *d32);
	int		set(uint c,uint pos,uint len,float *dfl);
	int		set(uint c,uint pos,uint len,double *ddb);
	int		set(uint c,uint pos,uint len,CSample *s);
	int		set(uint pos,uint len,CSample *s);
	int		set(uint c,uint pos,uint l,CSample *s,uint sc);

	int		move(uint dest,uint src,int len);

/*
	int		insert(uint pos,uint len,char*);
	int		insert(uint pos,uint len,short*);
	int		insert(uint pos,uint len,int*);
	int		insert(uint pos,uint len,float*);
	int		insert(uint pos,uint len,double*);
*/
	int		insert(uint pos,uint len,CSample *s);
	int		insert(uint pos,CSample *s);

	int		cut(uint pos,uint len,CSample *s = NULL);

	int		loadWave(char *fn);
	int		saveWave(char *fn);

	void		setSample(CSample *s);

	void		equalize(CSample *s);

protected:
	int		dsp_speed;
	int		dsp_stereo;
	int		dsp_samplesize;
	int		buffer_size;
	void		*buffer,*lb,*rb;
	uint		spb;
	int		audio;
	QString		audio_dev;
	uint		start_pos,end_pos,current_pos;

public:
	int		play();
	int		play(double,double);
	int		play(uint,uint);
	int		playback();
	int		playback(double,double);
	int		playback(uint,uint);
	int		playbackevent();
	void		stop();
	uint		currentPlayPos();
	double		currentPlayTime();
	int		playing();

	QString		getAudioDevice();
	void		setAudioDevice(QString);

};



#endif

