#include "ksoundwidget.h"
#include "ksoundwidget.moc"
#include <stdlib.h>
#include <math.h>

#include <qcursor.h>

KSoundWidget::KSoundWidget(QWidget *parent) :
	KDNDWidget(parent) ,
	CSample(2)
{
	setup();
}

KSoundWidget::KSoundWidget(QWidget *parent,int c,uint mode,int size = 1) : 
	KDNDWidget(parent) ,
	CSample(c,mode,size)
{
	setup();
}

KSoundWidget::KSoundWidget(QWidget *parent,CSample *s) :
	KDNDWidget(parent) ,
	CSample(s)
{
	setup();
}

KSoundWidget::~KSoundWidget()
{
	delete dnddz;
}

void	KSoundWidget::droped(KDNDDropZone *dz)
{
	const char	*d;
	char		*dd;
	int		s,t,x,y,pos;
	QPoint		g,l;

	d = dz->getData();
	s = dz->getDataSize();
	t = dz->getDataType();
	x = dz->getMouseX();
	y = dz->getMouseY();

	g.setX(x);
	g.setY(y);

	l = mapFromGlobal(g);

	x = l.x();
	y = l.y();

	printf("droped: size=%i type=%i pos=%i\n",s,t,x);

	switch(t)
	{
		case DndRawData:
			printf("Got RawData\n");
		break;
		case DndURL:
			dd = strdup(d);
			pos = MAX(0,MIN((int)getLength(),screen2sample(x)));
			printf("\nGot URL=%s  pos=%i\n\n",dd,pos);
			emit gotDndURL(dd,pos);
			free(dd);
		break;
	}
}

void	KSoundWidget::setup()
{
	QBitmap		&br = zoom_bitmap,&mr = zoom_mask;

	dnddz = new KDNDDropZone(this,DndRawData);

	connect(dnddz,SIGNAL(dropAction(KDNDDropZone*)),this,SLOT(droped(KDNDDropZone*)));

#include "pics/zoom.xbm"
#include "pics/zoom_mask.xbm"

	setMinimumSize(48,48);

	disp_start = 0;
	disp_zoom = 0;
	disp_mode = 0;

	sel_start = -1;
	sel_end = -1;
	sel_mode = 0;
	zsel_start = -1;
	zsel_end = -1;

	pointer = 0;

	scroll = 0;


	zoom_bitmap.QBitmap(zoom_width,zoom_height,(const uchar*)zoom_bits,TRUE);
	zoom_mask.QBitmap(zoom_mask_width,zoom_mask_height,(const uchar*)zoom_mask_bits,TRUE);
	zoom_cursor.QCursor(br,mr,3,7);

	disp_zoom = maxZoom();
	//printf("Zoom: %i %i\n",len,disp_zoom);

	painter = new QPainter(this);

	setMode(SM_SELECT);
}

void	KSoundWidget::getColors(QColor *color)
{
	color[0] = bgcolor;
	color[1] = fgcolor;
	color[2] = nocolor;
	color[3] = midlinecolor;
	color[4] = devidecolor;
	color[5] = sbgcolor;
	color[6] = sfgcolor;
	color[7] = smidlinecolor;
	color[8] = pointercolor;
	color[9] = spointercolor;
	color[10] = zoomcolor;
}

void	KSoundWidget::setColors(QColor *color)
{
	bgcolor = color[0];
	fgcolor = color[1];
	nocolor = color[2];
	midlinecolor = color[3];
	devidecolor = color[4];
	sbgcolor = color[5];
	sfgcolor = color[6];
	smidlinecolor = color[7];
	pointercolor = color[8];
	spointercolor = color[9];
	zoomcolor = color[10];

	layout();
}

int	KSoundWidget::sample2screen(int samplepos)
{
	if(samplepos < 0 || samplepos > (int)len) return -1;
	return (samplepos - disp_start) / (1 << disp_zoom);
}

int	KSoundWidget::screen2sample(int screenpos)
{
	return disp_start + screenpos * (1 << disp_zoom);
}

void	KSoundWidget::paintEvent(QPaintEvent *pe)
{
	if(pe == NULL) return;
	//draw(pe->rect().left() - 1,pe->rect().right() + 1);
	draw();
}

void	KSoundWidget::resizeEvent(QResizeEvent *re)
{
	if(re == NULL) return;
	//printf("resize\n");
	if(disp_zoom > maxZoom())
	{
		//printf("Zoom: %i\n",maxZoom());
		setDisplayZoom(maxZoom());
	}
	emit changedDisplayZoom(getDisplayZoom());
}

void	KSoundWidget::mousePressEvent(QMouseEvent *me)
{
	if((me->state() & MidButton) == MidButton) return;

	sel_on = TRUE;
	fm = lm = mr = ml = me->x();

	if(sel_mode == SM_SELECT)
	{
		draw(sample2screen(sel_start) - 1,sample2screen(sel_end) + 1,0); 
		if(screen2sample(fm) > (int)len) fm = lm = ml = mr = sample2screen(len);
		if(screen2sample(fm) < 0) fm = lm = ml = mr = sample2screen(0);
	}
	else if(sel_mode == SM_ZOOM)
	{
	}
}

void	KSoundWidget::mouseReleaseEvent(QMouseEvent *me)
{
	if((me->state() & MidButton) == MidButton) return;

	sel_on = FALSE;
	scroll = 0;
	//printf("scroll end\n");
	killTimers();
	if(sel_mode == SM_SELECT)
	{
		//printf("%i %i\n",sel_start,sel_end);
		if(fm == me->x())
		{
			sel_start = sel_end = -1;
			pointer = screen2sample(fm);
		}
		else
		{
			if(me->x() < 0) sel_start = 0;
			if(sel_start < 0) sel_start = 0;
			if(me->x() > sample2screen(len)) sel_end = len;
			if(sel_end > (int)len) sel_end = len;
		}
		//printf("%i %i\n",sel_start,sel_end);
		draw();
	}
	else if(sel_mode == SM_ZOOM)
	{
	}
//	draw();
}

#include "pics/dnd.xpm"

void	KSoundWidget::drop()
{
	QPixmap		dndpm(dnd_xpm);
	KDNDIcon	dndi(dndpm,1,1);
	char		*s;
	int		size,typesize,t;

	typesize = sizeof(float);

	size = sizeof(int) * (1 + 1);
	size += getNChannel() * getLength() * typesize;

	if((s = (char*)malloc(size)) == NULL) return;

	*((int*)s + 0) = (int)getNChannel();
	*((int*)s + 1) = (int)getLength();

	for(t = 0;t < getNChannel();t++)
	{
		get(t,0,getLength(),(float*)((int*)s + 2) + (t * getLength()));
	}
	startDrag(&dndi,s,size,DndRawData,0,0);
}

void	KSoundWidget::dndMouseMoveEvent(QMouseEvent *me)
{
	int		nm;

	if((me->state() & MidButton) == MidButton)
	{
		// causes a sigfault
		//drop();

		return;
	}

	if(!sel_on) return;
	nm = me->x();

	if(nm < 0)
	{
		if(screen2sample(nm) > 0)
		{
			if(!scroll) startTimer(150);
			scroll = nm;
			return;
		}
		if(scroll) killTimers();
		scroll = 0;
	}
	else if(nm >= width())
	{
		if(screen2sample(nm) < (int)len)
		{
			if(!scroll) startTimer(150);
			scroll = nm - width();
			return;
		}
		if(scroll) killTimers();
		scroll = 0;
	}
	else
	{
		if(scroll) killTimers();
		scroll = 0;
	}

	if(nm < sample2screen(0)) nm = sample2screen(0);
	if(nm > sample2screen(len)) nm = sample2screen(len);

	if(nm > fm)
	{
		ml = fm;
		mr = nm;
	}
	else
	{
		ml = nm;
		mr = fm;
	}

	if(sel_mode == SM_SELECT)
	{
		// if(sel_start >= screen2sample(0) || sel_start == -1)
			sel_start = screen2sample(ml);
		// if(sel_end <= screen2sample(width() - 1) || sel_end == -1)
			sel_end = screen2sample(mr);
		if(lm > nm)
			draw(nm - 2,lm + 2);
		else
			draw(lm - 2,nm + 2);
	}
	else if(sel_mode == SM_ZOOM)
	{
	}

	lm = nm;
}

void	KSoundWidget::timerEvent(QTimerEvent *te)
{
	int		nm = 0;

	if(te == NULL) return;
	//printf("scroll %i\n",scroll);

	scroll *= 2;

	if(sel_mode == SM_SELECT)
	{
		if(scroll < 0)
		{
			nm = 0;
		}
		else if(scroll > 0)
		{
			nm = width();
			if(screen2sample(nm) > (int)len) nm = sample2screen(len);
		}
		if(fm == sample2screen(sel_start))
		{
			if(!setDispStart(disp_start + scroll * (1 << disp_zoom)))
			{
				scroll = 0;
				killTimers();
				return;
			}
			fm = sample2screen(sel_start);
		}
		else if(fm == sample2screen(sel_end))
		{
			if(!setDispStart(disp_start + scroll * (1 << disp_zoom)))
			{
				scroll = 0;
				killTimers();
				return;
			}
			fm = sample2screen(sel_end);
		}

		if(nm > fm)
		{
			ml = fm;
			mr = nm;
		}
		else
		{
			ml = nm;
			mr = fm;
		}

		sel_start = screen2sample(ml);
		if(sel_start < 0) sel_start = 0;
		sel_end = screen2sample(mr);
		if(sel_end > (int)len) sel_end = (int)len;

		lm = nm;

		draw();
	}
	else if(sel_mode == SM_ZOOM)
	{
	}
	emit changedDisplayStart(getDisplayStart());
	draw();
}

void	KSoundWidget::layout()
{
	draw();
}

void	KSoundWidget::draw(int l,int r,int s)
{
	int		w,h,y,c,t;
	int		a,z,zs;
	int		i,ch;
	double		min,max,d = 0,lmin,lmax;

	w = width();
	h = height();

	if((c = getNChannel()) == 0) return;

	ch = (h - c) / c;

	zs = (1 << (disp_zoom / 2));

	for(t = 0;t < c;t++)
	{
		y = (ch / 2) + ch * t + t;

		lmin = lmax = 0;

		for(i = MAX(0,l);i < MIN(r,width());i++)
		{
			min = 2;
			max = -2;
			for(z = 0;z <= (1 << disp_zoom);z+=zs)
			{
				a = disp_start + i * (1L << disp_zoom) + z;
				if(a >= (int)len)
					break;
				if(isType8())
					d = CHAR2D(data[t].d8->at(a));
				else if(isType16())
					d = SHORT2D(data[t].d16->at(a));
				else if(isType32())
					d = INT2D(data[t].d32->at(a));
				else if(isTypeFL())
					d = FLOAT2D(data[t].dfl->at(a));
				else if(isTypeDB())
					d = data[t].ddb->at(a);
				d /= -2;
				min = MIN(min,d);
				max = MAX(max,d);
			}
			if(min == 2 || max == -2)
			{
				painter->setPen(nocolor);
				painter->drawLine(i,t * ch + t,i,(t + 1) * ch + t);
				painter->setPen(midlinecolor);
				painter->drawPoint(i,y);
			}
			else
			{
				d = min;
				if(min > lmax) min = lmax;
				lmin = d;
				d = max;
				if(max < lmin) max = lmin;
				lmax = d;
				if(s)	painter->setPen(sbgcolor);
				else	painter->setPen(bgcolor);
				painter->drawLine(i,t * ch + t,i,(int)(min * (double)(ch - 5)) + y - 1 + 1);
				painter->drawLine(i,(int)(max * (double)(ch - 5)) + y + 1 + t,i,(t + 1) * ch + t);
				if(s)	painter->setPen(sfgcolor);
				else	painter->setPen(fgcolor);
				painter->drawLine(i,
						(int)(min * (double)(ch - 5)) + y + t,
						  i,
						(int)(max * (double)(ch - 5)) + y + t);
				if(fabs(min) > 0.01 && fabs(max) > 0.01)
				{
					if(s)	painter->setPen(smidlinecolor);
					else	painter->setPen(midlinecolor);
					painter->drawPoint(i,y);
				}
			}
			if(i == sample2screen(pointer))
			{
				painter->setPen(pointercolor);
				painter->drawLine(i,t * ch + t,i,(t + 1) * ch + t);
			}
		}
		painter->setPen(zoomcolor);
		a = sample2screen(zsel_start);
		if(a > 0 && a < w) painter->drawLine(a,0,a,h);
		a = sample2screen(zsel_end);
		if(a > 0 && a < w) painter->drawLine(a,0,a,h);

		if(t + 1 < c)
		{
			painter->setPen(devidecolor);
			painter->drawLine(l,(t + 1) * ch + t,r,(t + 1) * ch + t);
		}
	}
}

void	KSoundWidget::draw()
{
	draw(0,width());
}

void	KSoundWidget::draw(int l,int r)
{
	int		sl,sr;

	if(l < 0) l = 0;
	if(l > width()) return;
	if(r > width()) r = width();
	if(r < 0) return;

	sl = sample2screen(sel_start);
	sr = sample2screen(sel_end);

	//printf("draw(%i,%i): %i %i %i %i\n",l,r,l,sl,sr,r);
	if(sel_start < 0 || sel_end < 0)
	{
		//printf("LINE %i\n",__LINE__);
		draw(l,r,0);
		return;
	}
	if(sr < 0 || sl > width())
	{
		//printf("LINE %i\n",__LINE__);
		draw(l,r,0);
		return;
	}
	if(sl - 1> l && sl - 1<= r)
	{
		//printf("LINE %i\n",__LINE__);
		draw(l,sl - 1,0);
	}
	if(sl <= l && sr >= r)
	{
		//printf("LINE %i\n",__LINE__);
		draw(l,r,1);
	}
	else if(sl <= l && sr <= r && sr >= l)
	{
		//printf("LINE %i\n",__LINE__);
		draw(l,sr,1);
	}
 	else if(sl >= l && sr >= r && sl <= r)
	{
		//printf("LINE %i\n",__LINE__);
		draw(sl,r,1);
	}
	else if(sl >= l && sr <= r)
	{
		//printf("LINE %i\n",__LINE__);
		draw(sl,sr,1);
	}
	if(sr + 1 < r && sr + 1>= l)
	{
		//printf("LINE %i\n",__LINE__);
		draw(sr + 1,r,0);
	}
	//printf("\n");
}

void	KSoundWidget::draw(int x)
{
	int		w,h,y,c,t,sl,sr,sel;
	int		ch;
	unsigned int	z;
	double		min,max,d = 0;

	w = width();
	h = height();

	c = getNChannel();

	sl = screen2sample(x);
	sr = screen2sample(x + 1);

	sel = (sl < sel_start && sr > sel_end);

	if(sel)
		painter->fillRect(x,0,x,h,sbgcolor);
	else
		painter->fillRect(x,0,x,h,bgcolor);

	for(t = 1;t < c;t++)
	{
		painter->setPen(devidecolor);
		painter->drawPoint(x,h / c * t);
	}

	ch = h / c;

	for(t = 0;t < c;t++)
	{
		if(sel)
			painter->setPen(smidlinecolor);
		else
			painter->setPen(midlinecolor);

		y = (h / c / 2) + h / c * t;
		painter->drawPoint(x,y);

		min = 2;
		max = -2;
		for(z = sl;(int)z <= sr;z++)
		{
			if(z >= len) break;
			if(isType8())
				d = CHAR2D(data[t].d8->at(z));
			else if(isType16())
				d = SHORT2D(data[t].d16->at(z));
			else if(isType32())
				d = INT2D(data[t].d32->at(z));
			else if(isTypeFL())
				d = FLOAT2D(data[t].dfl->at(z));
			else if(isTypeDB())
				d = data[t].ddb->at(z);
			d /= -2;
			min = MIN(min,d);
			max = MAX(max,d);
		}
		if(sel)
			painter->setPen(sfgcolor);
		else
			painter->setPen(fgcolor);
		if(!(min == 2 || max == -2))
			painter->drawLine(x,
				  (int)(min * (double)(ch - 5)) + y,
				  x,
				  (int)(max * (double)(ch - 5)) + y);
		if(x == sample2screen(pointer))
		{
			painter->setPen(pointercolor);
			painter->drawLine(x,t * ch + t,x,(t + 1) * ch + t);
		}
	}
}

void	KSoundWidget::setDisplayValues(int ds,int dz)
{
	int		redraw = FALSE;

	if(dz < 0) dz = 0;
	if(dz > maxZoom()) dz = maxZoom();

	if(disp_zoom != dz)
	{
		disp_zoom = dz;
		emit changedDisplayZoom(getDisplayZoom());
		redraw = TRUE;
	}

	if(ds + (width() - KSW_NAREA) * (1 << disp_zoom) > (int)len)
		ds = len - (width() - KSW_NAREA) * (1 << disp_zoom);
	if(ds < 0) ds = 0;

	if(labs(disp_start - ds) > 0)
	{
		disp_start = ds;
		emit changedDisplayStart(getDisplayStart());
		redraw = TRUE;
	}
	if(redraw) draw();
}

bool	KSoundWidget::setDispStart(int ds)
{
	if(ds + (width() - KSW_NAREA) * (1 << disp_zoom) > (int)len)
		ds = len - (width() - KSW_NAREA) * (1 << disp_zoom);
	if(ds < 0) ds = 0;
	if(disp_start != ds)
	{
		disp_start = ds;
		return TRUE;
	}
	return FALSE;
}

void	KSoundWidget::setDisplayStart(int ds)
{
	if(setDispStart(ds))
	{
		draw();
		emit changedDisplayStart(getDisplayStart());
	}
}

void	KSoundWidget::setDisplayZoom(int dz)
{
	int		ds;

	if(dz < 0) dz = 0;
	if(dz > maxZoom()) dz = maxZoom();

	//printf("Zoom: %i\n",dz);

	if(disp_zoom != dz)
	{
		disp_zoom = dz;
		ds = disp_start;
		setDisplayStart(disp_start);
		if(ds == disp_start) draw();
		emit changedDisplayZoom(getDisplayZoom());
	}
}

int	KSoundWidget::getDisplayStart()
{
	return disp_start;
}

int	KSoundWidget::getDisplayZoom()
{
	return disp_zoom;
}

void	KSoundWidget::resize(int size)
{
	CSample::resize(size);
}

void	KSoundWidget::resize(int w,int h)
{
	QWidget::resize(w,h);
}

void	KSoundWidget::setMode(int m)
{
	QCursor		c,&cr = c,&zcr = zoom_cursor;

	switch(m)
	{
		case SM_ZOOM:
			sel_mode = SM_ZOOM;
			setCursor(zcr);
		break;
		case SM_BUSY:
			sel_mode = SM_BUSY;
			c.setShape(3);
			setCursor(cr);
		break;
		case SM_SELECT:
		default:
			sel_mode = SM_SELECT;
			c.setShape(4);
			setCursor(cr);
		break;
	}
}

uint	KSoundWidget::getSelectionStart()
{
	return sel_start;
}

uint	KSoundWidget::getSelectionEnd()
{
	return sel_end;
}

void	KSoundWidget::setSelectionStart(int ss)
{
	if(ss == sel_start) return;
	if(ss < 0)
	{
		sel_start = -1;
		sel_end = -1;
	}
	else
	{
		sel_start = MIN(len,(uint)ss);
		if(sel_start > sel_end)
		{
			int	t;

			t = sel_start;
			sel_start = sel_end;
			sel_end = t;
		}
	}
	draw();
}

void	KSoundWidget::setSelectionEnd(int se)
{
	if(se == sel_end) return;
	if(se < 0)
	{
		sel_start = -1;
		sel_end = -1;
	}
	else
	{
		sel_start = MIN(len,(uint)se);
		if(sel_start > sel_end)
		{
			int	t;

			t = sel_start;
			sel_start = sel_end;
			sel_end = t;
		}
	}
	draw();
}


#ifdef KSW_TEST

#include <glib.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <kapp.h>

KApplication	*mykapp;

int	main(int argc,char **argv)
{
	KSoundWidget	*win;
	CSample		s(2,1,15000);
	short		sd[15000];
	int		t;

/*
	for(t = 0;t < 15000;t++)
	{
		sd[t] = (short)((sin((double)t / 500 * PI) + 
				cos((double)t / 50 * PI) + 
				sin((double)t / 5 * PI) / 5) * 12000);
	}
	s.set(0,0,15000,sd);
	s.set(1,0,15000,sd);
*/
	mykapp = new KApplication(argc,argv,"ksoundsys");

	win = new KSoundWidget(0);

	win->loadWave("samples/AllMyTroubles....wav");

	win->setMode(SM_ZOOM);

	mykapp->setMainWidget(win);

	win->resize(435,345);
	//win->setMaximumSize(435,345);
	//win->setMinimumSize(435,345);

	win->show();

	return mykapp->exec();
}

#endif // KSW_TEST

