#ifndef __KSOUNDWIDGET__
#define __KSOUNDWIDGET__

#include <qwidget.h>
#include <qpainter.h>
#include <qcursor.h>
#include <qbitmap.h>

#include <drag.h>

#include "csample.h"

#define KSW_NAREA	32

enum	{
		SM_SELECT,
		SM_ZOOM,
		SM_BUSY,
	};

class KSoundWidget : public KDNDWidget , public CSample
{
	Q_OBJECT
	friend class	KSoundWidgetPro;

protected:
	int		disp_start,disp_mode;
	int		disp_zoom;
	int		disp_start_old;
	int		sel_start,sel_end,sel_mode;
	int		zsel_start,zsel_end;
	int		pointer;

	QPainter	*painter;
	QBitmap		zoom_bitmap,zoom_mask;
	QCursor		zoom_cursor;

	KDNDDropZone	*dnddz;

	QColor		bgcolor,fgcolor,nocolor,midlinecolor,devidecolor;
	QColor		sbgcolor,sfgcolor,smidlinecolor;
	QColor		pointercolor,spointercolor;
	QColor		zoomcolor;

	int		sel_on;
	int		fm,lm,ml,mr;
	int		xl,xr,lx,fx;
	int		scroll;


protected:
	void		layout();

	int		screen2sample(int);
	int		sample2screen(int);

	int		calcZoom(int l)
	{
		int		t;

		l /= width();
		for(t = 0;l > 0;t++) l >>= 1;

		return t;
	}
	int		calcZoom(int a,int b)
	{
		return calcZoom(b - a);
	}
	int		maxZoom()
	{
		return calcZoom(len);
	}

	bool		setDispStart(int ds);

	void		paintEvent(QPaintEvent *);
	void		resizeEvent(QResizeEvent *);
	void		mousePressEvent(QMouseEvent *);
	void		mouseReleaseEvent(QMouseEvent *);
	void		dndMouseMoveEvent(QMouseEvent *);
	void		timerEvent(QTimerEvent *);

public:

	KSoundWidget(QWidget *parent);
	KSoundWidget(QWidget *parent,int c,uint mode,int size = 1);
	KSoundWidget(QWidget *parent,CSample *s);
	~KSoundWidget();

	void		setup();
	void		getColors(QColor *color);
	void		setColors(QColor *color);

	void		draw();
	void		draw(int);
	void		draw(int,int);
	void		draw(int,int,int);

	void		resize(int size);
	void		resize(int w,int h);

	unsigned int	getSelectionStart();
	unsigned int	getSelectionEnd();
	void		setSelectionStart(int);
	void		setSelectionEnd(int);

	void		setDisplayValues(int ds,int dz);
	void		setDisplayStart(int ds);
	void		setDisplayZoom(int dz);
	int		getDisplayZoom();
	int		getDisplayStart();

	int		getPointer() { return pointer;};

	void		setMode(int m);
	int		getMode() { return sel_mode;};

	CSample		*getSample()
	{
		return	this;
	};

	int		getSampleRate()
	{
		return CSample::getRate();
	};

	// bool		scroll(int pixel);

	void		drop();

private slots:
	void		droped(KDNDDropZone*);

signals:

	void		changedDisplayStart(int);
	void		changedDisplayZoom(int);
	void		gotDndURL(char *,int);
};

#endif // KSOUNDWIDGET

