#include "ksoundwidgetpro.h"
#include "ksoundwidgetpro.moc"

#include <qpixmap.h>

#include <stdlib.h>

KSoundWidgetPro::KSoundWidgetPro(QWidget *parent,KApplication *ka) : 
	QWidget(parent,NULL)
{
	nchannel = 0;

	app = ka;

	createWidgets();
}

KSoundWidgetPro::KSoundWidgetPro(QWidget *parent,CSample *s,KApplication *ka) : 
	QWidget(parent,NULL)
{
	nchannel = s->getNChannel();

	app = ka;

	createWidgets(s);
}

KSoundWidgetPro::~KSoundWidgetPro()
{
	int		t;

	delete qsb;
	delete zoom_in;
	delete zoom_out;

	for(t = 0;t < ksw->getNChannel() * 2;t++)
	{
		delete ksb[t];
	}
	ksb.resize(0);

	delete ksw;
	delete ktb;
}

void	KSoundWidgetPro::createWidgets(CSample *s = NULL)
{
	int		w,h,t;
	QPixmap		&pi = pm_zoom_in;
	QPixmap		&po = pm_zoom_out;
	QString		str;

	w = width();
	h = height();

	qsb = new QScrollBar(this);

	qsb->setOrientation(QScrollBar::Horizontal);

	ktb = new KTimeBar(this);
	if(s)
	{
		ktb->setRange(0,100000);
		ktb->setTimeRange(0,s->getLength() * 1000 / s->getRate());
	}
	else
	{
		ktb->setRange(0,0);
		ktb->setTimeRange(0,0);
	}
	ktb->setMinStep(1);

	if(s) ksw = new KSoundWidget(this,s);
	else ksw = new KSoundWidget(this);

	//printf("rate=%i LINE %i\n",ksw->getRate(),__LINE__);
	//printf("rate=%i LINE %i\n",s->getRate(),__LINE__);

	ksw->setMode(SM_SELECT);

	str = app->kde_datadir() + "/ksoundsys/pics/zoom_in.xpm";
	//printf("%s\n",str.data());
	pm_zoom_in.QPixmap(str);
	zoom_in = new QPushButton(this);
	zoom_in->setPixmap(pi);

	str = app->kde_datadir() + "/ksoundsys/pics/zoom_out.xpm";
	//printf("%s\n",str.data());
	pm_zoom_out.QPixmap(str);
	zoom_out = new QPushButton(this);
	zoom_out->setPixmap(po);

	nchannel = ksw->getNChannel();
	ksb.resize(2 * nchannel);
	// printf("nchannel:%i\n",ksw->getNChannel());
	// printf("nchannel:%i\n",s->getNChannel());
	for(t = 0; t < nchannel;t++)
	{
		ksb[t * 2] = new KScaleBar(this,LEFT_SIDED|ZERO_MID);
		ksb[t * 2]->setMode(LEFT_SIDED|ZERO_MID);
		ksb[t * 2]->setRange(1,1);
		ksb[t * 2]->setNMarks(5);
		ksb[t * 2]->setFormatString("%1.1f");

		ksb[t * 2 + 1] = new KScaleBar(this,RIGHT_SIDED|ZERO_MID);
		ksb[t * 2 + 1]->setMode(RIGHT_SIDED|ZERO_MID);
		ksb[t * 2 + 1]->setRange(1,1);
		ksb[t * 2 + 1]->setNMarks(5);
		ksb[t * 2 + 1]->setFormatString("%1.1f");
	}
	

	connect(qsb,SIGNAL(valueChanged(int)),SLOT(changedScroll(int)));
	connect(ksw,SIGNAL(changedDisplayStart(int)),SLOT(changedDispStart(int)));
	connect(ksw,SIGNAL(changedDisplayZoom(int)),SLOT(changedDispZoom(int)));
	connect(zoom_in,SIGNAL(released()),SLOT(clickedZoomIn()));
	connect(zoom_out,SIGNAL(released()),SLOT(clickedZoomOut()));
	connect(ksw,SIGNAL(gotDndURL(char*,int)),SLOT(gotDndURL(char*,int)));
}

void	KSoundWidgetPro::layout()
{
	int		w,h,t;

	w = width();
	h = height();

	ksw->QWidget::move(32,18);
	ksw->resize(w - 64,h - 16 - 18);

	qsb->move(0,h - 16);
	qsb->resize(w - 32,16);

	ktb->move(32,0);
	ktb->resize(w - 32,18);

	zoom_in->move(w - 32,h - 16);
	zoom_in->resize(16,16);

	zoom_out->move(w - 16,h - 16);
	zoom_out->resize(16,16);

	if(ksw->getDisplayZoom() > 0)
		qsb->setRange(0,MAX(0,(int)ksw->getLength() 
				/ ksw->getDisplayZoom() 
				- ksw->width()));

	//printf("nchannel:%i\n",ksw->getNChannel());
	for(t = 0; t < ksw->getNChannel();t++)
	{
		ksb[t * 2]->move(0,18 + t * (h - 16 - 18) / ksw->getNChannel());
		ksb[t * 2]->resize(32,(h - 16 - 18) / ksw->getNChannel());
		ksb[t * 2 + 1]->move(w - 32,18 + t * (h - 16 - 18)  / ksw->getNChannel());
		ksb[t * 2 + 1]->resize(32,(h - 16 - 18) / ksw->getNChannel());
	}
}

void	KSoundWidgetPro::paintEvent(QPaintEvent *pe)
{
	int		min,max,rate;

	if(pe == NULL) return;

	layout();

	rate = ksw->getRate();
	if(rate)
	{
		min = ksw->screen2sample(0) * 1000 / rate;
		max = ksw->screen2sample(ktb->width()) * 1000 / rate;
		ktb->setTimeRange(0,ksw->getLength() * 1000 / rate);
	}
	else min = max = 0;
	ktb->setRange(min,max);
}

void	KSoundWidgetPro::paint()
{
	int		min,max,rate;

	layout();

	rate = ksw->getRate();
	if(rate)
	{
		min = ksw->screen2sample(0) * 1000 / rate;
		max = ksw->screen2sample(ktb->width()) * 1000 / rate;
		ktb->setTimeRange(0,ksw->getLength() * 1000 / rate);
	}
	else min = max = 0;
	ktb->setRange(min,max);
	ksw->draw();
}

void	KSoundWidgetPro::changedScroll(int v)
{
	int		i;

	i = v * (1 << ksw->getDisplayZoom());
	ksw->setDisplayStart(i);
}

void	KSoundWidgetPro::changedDispStart(int v)
{
	int		i,min,max,rate;

	i = v / (1 << ksw->getDisplayZoom());
	qsb->setValue(i);

	rate = ksw->getSampleRate();
	if(rate)
	{
		min = ksw->screen2sample(0) * 1000 / rate;
		max = ksw->screen2sample(ktb->width()) * 1000 / rate;
		ktb->setTimeRange(0,ksw->getLength() * 1000 / rate);
	}
	else min = max = 0;
	ktb->setRange(min,max);
}

void	KSoundWidgetPro::changedDispZoom(int v)
{
	int		min,max,rate;

	qsb->setRange(0,MAX(0,(int)ksw->getLength() / (1 << v) - ksw->width() + KSW_NAREA));
	changedDispStart(ksw->getDisplayStart());

	rate = ksw->getRate();
	if(rate)
	{
		min = ksw->screen2sample(0) * 1000 / rate;
		max = ksw->screen2sample(ktb->width()) * 1000 / rate;
		ktb->setTimeRange(0,ksw->getLength() * 1000 / rate);
	}
	else min = max = 0;
	ktb->setRange(min,max);
}

void	KSoundWidgetPro::clickedZoomIn()
{
	ksw->setDisplayZoom(ksw->getDisplayZoom() - 1);
}

void	KSoundWidgetPro::clickedZoomOut()
{
	ksw->setDisplayZoom(ksw->getDisplayZoom() + 1);
}

void	KSoundWidgetPro::gotDndURL(char *url,int pos)
{
	emit gotURL(url,pos);
}

void	KSoundWidgetPro::setMode(int m)
{
	ksw->setMode(m);
}

void	KSoundWidgetPro::setSample(CSample *s)
{
	int		t;

	for(t = 0;t < nchannel * 2;t++)
	{
		delete ksb[t];
	}

	ksw->setSample(s);

	nchannel = ksw->getNChannel();

	ksb.resize(2 * nchannel);
	for(t = 0; t < nchannel;t++)
	{
		ksb[t * 2] = new KScaleBar(this,LEFT_SIDED|ZERO_MID);
		ksb[t * 2]->setMode(LEFT_SIDED|ZERO_MID);
		ksb[t * 2]->setRange(1,1);
		ksb[t * 2]->setNMarks(5);
		ksb[t * 2]->setFormatString("%1.1f\0");
		ksb[t * 2]->show();

		ksb[t * 2 + 1] = new KScaleBar(this,RIGHT_SIDED|ZERO_MID);
		ksb[t * 2 + 1]->setMode(RIGHT_SIDED|ZERO_MID);
		ksb[t * 2 + 1]->setRange(1,1);
		ksb[t * 2 + 1]->setNMarks(5);
		ksb[t * 2 + 1]->setFormatString("%1.1f\0");
		ksb[t * 2 + 1]->show();
	}

	layout();
}

void	KSoundWidgetPro::setSample()
{
	int		t;

	for(t = 0;t < nchannel * 2;t++)
	{
		delete ksb[t];
	}

	//ksw->setSample(s);

	nchannel = ksw->getNChannel();

	ksb.resize(2 * nchannel);
	for(t = 0; t < nchannel;t++)
	{
		ksb[t * 2] = new KScaleBar(this,LEFT_SIDED|ZERO_MID);
		ksb[t * 2]->setMode(LEFT_SIDED|ZERO_MID);
		ksb[t * 2]->setRange(1,1);
		ksb[t * 2]->setNMarks(5);
		ksb[t * 2]->setFormatString("%1.1f\0");
		ksb[t * 2]->show();

		ksb[t * 2 + 1] = new KScaleBar(this,RIGHT_SIDED|ZERO_MID);
		ksb[t * 2 + 1]->setMode(RIGHT_SIDED|ZERO_MID);
		ksb[t * 2 + 1]->setRange(1,1);
		ksb[t * 2 + 1]->setNMarks(5);
		ksb[t * 2 + 1]->setFormatString("%1.1f\0");
		ksb[t * 2 + 1]->show();
	}

	layout();
}

void	KSoundWidgetPro::getColors(QColor *color)
{
	ksw->getColors(color);
}

void	KSoundWidgetPro::setColors(QColor *color)
{
	ksw->setColors(color);
}




#ifdef KSWP_TEST

#include <glib.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <kapp.h>

KApplication	*mykapp;

int	main(int argc,char **argv)
{
	KSoundWidgetPro	*win;
	CSample		s;

	mykapp = new KApplication(argc,argv,"ksoundsys");

	s.CSample((uint)0);
	s.loadWave("samples/start.wav");
	//printf("rate=%i\n",s.getRate());
	printf("channel=%i Line %i\n",s.getNChannel(),__LINE__);

	win = new KSoundWidgetPro(0,&s);

//	if(argc > 1) win->loadWave(argv[1]);

//	win = new KSoundWidgetPro(0);

	win->setMode(SM_SELECT);
//	win->setDisplayZoom(15000);

	mykapp->setMainWidget(win);

	win->resize(435,345);
	//win->setMaximumSize(435,345);
	//win->setMinimumSize(435,345);

	win->show();

	return mykapp->exec();
}

#endif //KSWP__TEST


