#ifndef __KSS_DIALOG__
#define __KSS_DIALOG__

#include <qdialog.h>
#include <qcombo.h>
#include <qbutton.h>
#include <qchkbox.h>
#include <qpushbt.h>
#include <qlined.h> 
#include <qvalidator.h> 
#include <qradiobt.h> 
#include <qframe.h> 
#include <qlabel.h> 
#include <qstring.h> 
#include <qapp.h> 

#include <kcolorbtn.h> 
#include <kconfig.h> 
#include <klocale.h> 
#include <kprogress.h> 
#include <kcombo.h>

#include "KGraphEdit.h"

class	namedialog : public QDialog
{
Q_OBJECT

protected:
	QPushButton	*ok,*cancel;
	QLineEdit	*line;

public:
	namedialog(QWidget *parent,const char *name);
	~namedialog();

	QString		getName();

};

class	kssdialog : public QDialog
{
Q_OBJECT

private:
protected:
	QPushButton	*ok,*cancel,*save,*del;
	//KCombo		*list;
	QComboBox	*list;

protected:
	QString		profile;
	KConfig		*config;
	KLocale		*locale;
	QStrList	*strlist;

public:
	kssdialog(QWidget *parent,const char *name,KConfig *c,QString pf);
	~kssdialog();

	void		resize(int w,int h);
	void		addEntry(QString);

protected:
	QString		getName(char *);

public slots:
	virtual void	loadProfile(const char*) = 0;
	virtual void	saveProfile(const char*) = 0;
	virtual void	deleteProfile() = 0;

	void		saveProfileInt();

signals:
	void		activated(const char*);

};

class	envdialog : public kssdialog
{
Q_OBJECT

protected:
	KGraphEdit	*kge;

public:
	envdialog(QWidget *parent,const char *name,KConfig *c);
	~envdialog();

	QArray<double>	getPoints();

public slots:

	virtual void	loadProfile(const char*);
	virtual void	saveProfile(const char*);
	virtual void	deleteProfile();

};

class	distdialog : public kssdialog
{
Q_OBJECT

protected:
	KGraphEdit	*kge;
	QCheckBox	*qcb;

public:
	distdialog(QWidget *parent,const char *name,KConfig *c);
	~distdialog();

	QArray<double>	getPoints();
	int		symmetric();

public slots:

	virtual void	loadProfile(const char*);
	virtual void	saveProfile(const char*);
	virtual void	deleteProfile();

};

class	dphpdialog : public kssdialog
{
Q_OBJECT

protected:
	double			r,c;
	QLineEdit		*rline,*cline;
	QLabel			*rlabel,*clabel;
	QDoubleValidator	*dblv;

public:
	dphpdialog(QWidget *parent,const char *name,KConfig *c);
	~dphpdialog();

	double		getR();
	double		getC();


public slots:

	virtual void	loadProfile(const char*);
	virtual void	saveProfile(const char*);
	virtual void	deleteProfile();

};

class	voldialog : public kssdialog
{
Q_OBJECT

protected:
	double			vol;
	QLineEdit		*volline;
	QLabel			*vollabel;
	QDoubleValidator	*dblv;

public:
	voldialog(QWidget *parent,const char *name,KConfig *c);
	~voldialog();

	double		getVol();

public slots:

	virtual void	loadProfile(const char*);
	virtual void	saveProfile(const char*);
	virtual void	deleteProfile();

};

class	timedialog : public kssdialog
{
Q_OBJECT

protected:
	double			sec;
	QLineEdit		*secline;
	QLabel			*seclabel;
	QDoubleValidator	*dblv;

public:
	timedialog(QWidget *parent,const char *name,KConfig *c);
	~timedialog();

	double		getSeconds();

public slots:

	virtual void	loadProfile(const char*);
	virtual void	saveProfile(const char*);
	virtual void	deleteProfile();

};

class	echodialog : public kssdialog
{
Q_OBJECT

protected:
	double			vol,delay,release;
	QLineEdit		*volline,*delayline,*releaseline;
	QLabel			*vollabel,*delaylabel,*releaselabel;
	QDoubleValidator	*volv,*delayv,*releasev;

public:
	echodialog(QWidget *parent,const char *name,KConfig *c);
	~echodialog();

	double		getDelay();
	double		getVolume();
	double		getRelease();

public slots:

	virtual void	loadProfile(const char*);
	virtual void	saveProfile(const char*);
	virtual void	deleteProfile();

};

class	fuzzdialog : public kssdialog
{
Q_OBJECT

protected:
	double			vol,cut;
	int			side;
	QLineEdit		*volline,*cutline,*sideline;
	QLabel			*vollabel,*cutlabel,*sidelabel;
	QDoubleValidator	*volv,*cutv,*sidev;

public:
	fuzzdialog(QWidget *parent,const char *name,KConfig *c);
	~fuzzdialog();

	double		getCut();
	double		getVolume();
	int		getSide();

public slots:

	virtual void	loadProfile(const char*);
	virtual void	saveProfile(const char*);
	virtual void	deleteProfile();

};

class	formatdialog : public kssdialog
{
Q_OBJECT

protected:
	int			samplerate,type,nchannel;
	QRadioButton		*I8,*I16,*I32,*FL,*DB;
	QFrame			*typeframe;
	QLabel			*sampleratelabel;
	QLineEdit		*samplerateline;
	QDoubleValidator	*samplerateval;

private slots:
	void		type8Clicked();
	void		type16Clicked();
	void		type32Clicked();
	void		typeFLClicked();
	void		typeDBClicked();

public slots:

	void		loadProfile(const char*);
	void		saveProfile(const char*);
	void		deleteProfile();

public:
	formatdialog(QWidget *parent,const char *name,KConfig *c,int,int);
	~formatdialog();

	uint		getSampleRate();
	uint		getType();
};

class	newdialog : public kssdialog
{
Q_OBJECT

protected:
	int			samplerate,type,nchannel;
	QRadioButton		*I8,*I16,*I32,*FL,*DB;
	QFrame			*typeframe;
	QLabel			*sampleratelabel,*nchannellabel;
	QLineEdit		*samplerateline,*nchannelline;
	QDoubleValidator	*samplerateval,*nchannelval;

private slots:
	void		type8Clicked();
	void		type16Clicked();
	void		type32Clicked();
	void		typeFLClicked();
	void		typeDBClicked();

public:
	newdialog(QWidget *parent,const char *name,KConfig *c,uint,uint,uint);
	~newdialog();

	uint		getSampleRate();
	uint		getType();
	uint		getNChannel();

public slots:

	void		loadProfile(const char*);
	void		saveProfile(const char*);
	void		deleteProfile();

};


class	colordialog : public kssdialog
{
Q_OBJECT

protected:
	KColorButton		*colorbt[12];
	QLabel			*colorlabel[12];
public:
	colordialog(QWidget *parent,const char *name,KConfig *c,QColor *q[11]);
	~colordialog();

	void		getColors(QColor *c[11]);

public slots:

	void		loadProfile(const char*);
	void		saveProfile(const char*);
	void		deleteProfile();

};

class	noisegatedialog : public kssdialog
{
Q_OBJECT

protected:
	double			vol;
	QLineEdit		*volline;
	QLabel			*vollabel;
	QDoubleValidator	*dblv;

public:
	noisegatedialog(QWidget *parent,const char *name,KConfig *c);
	~noisegatedialog();

	double		getTreshhold();

public slots:

	virtual void	loadProfile(const char*);
	virtual void	saveProfile(const char*);
	virtual void	deleteProfile();

};

class	compressdialog : public kssdialog
{
Q_OBJECT

protected:
	double			level;
	QLineEdit		*levelline;
	QLabel			*levellabel;
	QDoubleValidator	*dblv;

public:
	compressdialog(QWidget *parent,const char *name,KConfig *c);
	~compressdialog();

	double		getLevel();

public slots:

	virtual void	loadProfile(const char*);
	virtual void	saveProfile(const char*);
	virtual void	deleteProfile();

};


// ******************************************************************************


class progressdialog : public QDialog
{
Q_OBJECT

protected:
	KProgress	*progress;
	QLabel		*label;
	QPushButton	*cancel;
	QApplication	*app;

public:
	progressdialog(QApplication*,QWidget *parent,const char *name);
	~progressdialog();

	void		setText(QString);
	void		setProgress(double);
	void		sync();

};

class optiondialog : public QDialog
{
Q_OBJECT

protected:
	QLabel		*dsplabel;
	QLineEdit	*dspline;
	QPushButton	*ok,*cancel;
	KConfig		*config;

public:
	optiondialog(QWidget *parent,const char *name,KConfig *c);
	~optiondialog();

	QString		getDSP();
	void		setDSP(QString);

public slots:

	void		okPressed();
};

#endif

