#include "ktimebar.h"
#include "ktimebar.moc"

#include <stdio.h>

#include <math.h>
#include <string.h>
#include <stdlib.h>

KTimeBar::KTimeBar(QWidget *parent = NULL) :
	QFrame(parent)
{
	QFont	f;
	int	s;

	min_value = 0;
	max_value = 100;
	nmarks = 10;
	minstep = 50;

	qp = new QPainter(this);

	f = qp->font();
	s = MAX(10,f.pointSize() / 2);
	//printf("%i\n",s);
	f.setPointSize(s);
	qp->setFont(f);

	format = "%h:%2m:%2s.%3u";
}

KTimeBar::~KTimeBar()
{
	delete qp;
}

void	KTimeBar::sprinttf(QString *d,QString f,int v)
{
	int	h,m,s,u;
	int	t,l;
	QString	fi,str;

	u = v % 1000;
	s = (v / 1000) % 60;
	m = (v / 60000) % 60;
	h = (v / 3600000);

	*d = "";

	for(t = 0;t < (int)f.length();t++)
	{
		if(f[t] != '%')
		{
			*d += f[t];
		}
		else
		{
			str = f.mid(t + 1,64);
			if((l = atoi(str.data())) != 0)
			{
				fi.sprintf("%%0%ii",l);
				t++;
			}
			else
			{
				fi.sprintf("%%i");
			}
			switch(f[++t])
			{
				case 'i':
					str.sprintf(fi,v);
				break;
				case 'u':
					str.sprintf(fi,u);
				break;
				case 's':
					str.sprintf(fi,s);
				break;
				case 'm':
					str.sprintf(fi,m);
				break;
				case 'h':
					str.sprintf(fi,h);
				break;
				default:
					str = f[t];
			}
			*d += str;
		}
	}
}

void	KTimeBar::draw()
{
	QColor		black;
	int		x,y,w,h;
	int		tw,th,d,n;
	double		ts,ss,offset,tc,sc;
	int		t,i;
	int		v;
	QString		str;
	QRect		qr;

	black.setRgb(0,0,0);

	setFrameStyle( QFrame::Panel | QFrame::Raised );
	setLineWidth( 2 );

	erase(0,0,width(),height());

	qp->setPen(black);

	sprinttf(&str,format,max_value);

	const QFontMetrics	&qfm = qp->fontMetrics();
	qr = qfm.boundingRect(str);

	tw = qr.width();
	th = qr.height();

	w = width();
	h = height();

	d = end_value - start_value;

	t = 0;
	ts = (double)minstep;
	ss = ts / (double)d * (double)w;

	while(ss < 5 || ss * 10 < tw + 16)
	{
		if(t & 1) ts = (double)minstep * pow(10,(double)(t / 2)) * 5;
		else ts = (double)minstep * pow(10,(double)(t / 2));
		ss = ts / (double)d * (double)w;
		t++;
	}

	offset = fmod((double)start_value,ts * 10);
	i = (int)((offset + ts - 1) / ts);
	if(i < 0) i += 10;
	//printf("offset=%f ts=%f i=%i\n",offset,ts * 10,i);

	offset = fmod((double)start_value,ts);

	//printf("offset=%f ts=%f ss=%f\n",offset,ts,ss);

	tc = -offset;
	if(tc < 0) tc += ts;
	sc = tc / (double)d * (double)w;

	n = w / (int)ss;

	for(t = 0;t < n;t++)
	{
		x = (int)sc;
		sc += ss;
		v = start_value + (int)tc;
		tc += ts;
		if(v > max_value)
		{
			x = (int)((double)(max_value - start_value) / (double)d * (double)width());
			sprinttf(&str,format,max_value);
			y = 2;
			w = width() - 12;
			h = height() - 8;
			qp->drawText(x + 2,y,w,h,
				AlignBottom|AlignLeft,
				str.data());
			y = height() - 12;
			w = 0;
			h = 11;
			qp->drawLine(x,y,x + w,y + h);
			break;
		}
		if((t + i) % 10 == 0)
		{
			int	xx;

			xx = (int)((double)(max_value - start_value) / (double)d * (double)width());
			sprinttf(&str,format,(int)v);
			if(xx - x > tw + 16) 
			{
				y = 2;
				w = width() - 12;
				h = height() - 8;
				qp->drawText(x + 2,y,w,h,
					AlignBottom|AlignLeft,
					str.data());
			}
			y = height() - 12;
			w = 0;
			h = 11;
			qp->drawLine(x,y,x + w,y + h);
		}
		else
		{
			w = 0;
			if((t + i + 5) % 10 == 0)
			{
				y = height() - 6;
				h = 5;
			}
			else
			{
				y = height() - 4;
				h = 3;
			}
			//printf("%i %i %i %i\n",x,y,w,h);
			qp->drawLine(x,y,x + w,y + h);
		}
	}
	//qp->end();
}

void	KTimeBar::paintEvent(QPaintEvent *pe)
{
	if(pe == NULL) return;
	draw();
}




#ifdef KTB_TEST

#include <glib.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <kapp.h>

KApplication	*mykapp;

int	main(int argc,char **argv)
{
	KTimeBar	*win;

	mykapp = new KApplication(argc,argv,"ksoundsys");


	win = new KTimeBar(0);

	win->setRange(0,((0 * 60 + 0) * 60 + 15) * 1000 + 0);

	mykapp->setMainWidget(win);

	win->resize(900,18);

	win->show();

	return mykapp->exec();
}

#endif //KSB__TEST



